#!/usr/bin/python

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys

sys.path.insert(0, '/usr/share/yum-cli')

import yum
import yum.Errors
import os
import output
from urlgrabber.progress import TextMeter
from yum.logger import Logger
from yum.packages import parsePackages
from yum.misc import getCacheDir
from optparse import OptionParser
from urlparse import urljoin

def initYum(source=False):
    my = yum.YumBase()
    my.doConfigSetup()
    my.conf.setConfigOption('uid', os.geteuid())
    if my.conf.getConfigOption('uid') != 0:
        cachedir = getCacheDir()
        if cachedir is None:
            print "Error: Could not make cachedir, exiting"
            sys.exit(50)
        my.repos.setCacheDir(cachedir)
    my.repos.setProgressBar(TextMeter(fo=sys.stdout))
    my.log = Logger(threshold=my.conf.getConfigOption('debuglevel'), 
    file_object =sys.stdout)
    my.repos.callback = output.CacheProgressCallback(my.log,
    my.errorlog, my.filelog)

    my.doRepoSetup()
    archlist = None
    if source:
        archlist = ['src']

    my.doSackSetup(archlist=archlist)
    return my

def parseArgs():
    usage = "usage: %s [options] package1 [package2] [package..]" % sys.argv[0]
    parser = OptionParser(usage=usage)
    parser.add_option("--destdir", default=".", dest="destdir",
      help='destination directory (defaults to current directory)')
    parser.add_option("--urls", default=False, dest="urls", action="store_true",
      help='just list the urls it would download instead of downloading')
    parser.add_option("--resolve", default=False, dest="resolve", action="store_true",
      help='resolve dependencies and download required packages')
    parser.add_option("--source", default=False, dest="source", action="store_true",
      help='operate on source packages')
    (opts, args) = parser.parse_args()
    if len(args) < 1: 
        parser.print_help()
        sys.exit(0)
    return (opts, args)

def main():
    (opts, args) = parseArgs()
    my = initYum(opts.source)
    avail = my.pkgSack.returnPackages()
    toDownload = []

    packages = args
    for pkg in packages:
        exactmatch, matched, unmatched = parsePackages(avail, [pkg])
        installable = yum.misc.unique(exactmatch + matched)
        if len(unmatched) > 0: # if we get back anything in unmatched, it fails
            my.errorlog(0, 'No Match for argument %s' % pkg)
            continue
        for newpkg in installable:
            toDownload.append(newpkg)

    if opts.source:
        toDownload = my.bestPackagesFromList(toDownload, 'src')
    else:
        toDownload = my.bestPackagesFromList(toDownload)
        
    # If the user supplies to --resolve flag, resolve dependencies for
    # all packages
    # note this might require root access because the headers need to be
    # downloaded into the cachedir (is there a way around this)
    if opts.resolve:
        my.doTsSetup()
        my.localPackages = []
        # Act as if we were to install the packages in toDownload
        for po in toDownload:
            my.tsInfo.addInstall(po)
            my.localPackages.append(po)
        # Resolve dependencies
        my.resolveDeps()
        # Add newly added packages to the toDownload list
        for pkg in my.tsInfo.getMembers():
            if not pkg in toDownload:
                toDownload.append(pkg)
        
    for pkg in toDownload:
        n,a,e,v,r = pkg.pkgtup
        packages =  my.pkgSack.searchNevra(n,e,v,r,a)
        for download in packages:
            repo = my.repos.getRepo(download.repoid)
            remote = download.returnSimple('relativepath')
            if opts.urls:
                url = urljoin(repo.urls[0],remote)
                my.log(0, '%s' % url)
                continue
            local = os.path.basename(remote)
            local = os.path.join(opts.destdir, local)
            if (os.path.exists(local) and 
                str(os.path.getsize(local)) == download.returnSimple('packagesize')):
                my.errorlog(0,"%s already exists and appears to be complete" % local)
                continue
            # Disable cache otherwise things won't download
            repo.cache = 0
            repo.get(relative=remote, local=local)

if __name__ == '__main__':
    main()
